# pylint: disable=no-self-use
from typing import Any

from winregistry import RegEntry, RegKey, WinRegistry, WinregType


class Keywords:
    def read_registry_key(
        self,
        key: str,
        key_wow64_32key: bool = False,
    ) -> RegKey:
        """Reading registry key"""
        with WinRegistry() as client:
            resp = client.read_key(key, key_wow64_32key)
        return resp

    def create_registry_key(
        self,
        key: str,
        key_wow64_32key: bool = False,
    ) -> None:
        """Creating registry key"""
        with WinRegistry() as client:
            client.create_key(key, key_wow64_32key)

    def delete_registry_key(
        self,
        key: str,
        key_wow64_32key: bool = False,
    ) -> None:
        """Deleting registry key"""
        with WinRegistry() as client:
            client.delete_key(key, key_wow64_32key)

    def read_registry_entry(
        self,
        key: str,
        value: Any,
        key_wow64_32key: bool = False,
    ) -> RegEntry:
        """Reading value from registry"""
        with WinRegistry() as client:
            resp = client.read_entry(key, value, key_wow64_32key)
        return resp

    def write_registry_entry(
        self,
        key: str,
        value: str,
        data: Any = None,
        reg_type: str = "REG_SZ",
        key_wow64_32key: bool = False,
    ) -> None:
        """Writing (or creating) data in value"""
        with WinRegistry() as client:
            client.write_entry(key, value, data, WinregType[reg_type], key_wow64_32key)

    def delete_registry_entry(
        self,
        key: str,
        value: str,
        key_wow64_32key: bool = False,
    ) -> None:
        """Deleting value from registry"""
        with WinRegistry() as client:
            client.delete_entry(key, value, key_wow64_32key)



import winreg
from enum import Enum, IntEnum


class ShortRootAlias(Enum):
    HKCR = "HKEY_CLASSES_ROOT"
    HKCU = "HKEY_CURRENT_USER"
    HKLM = "HKEY_LOCAL_MACHINE"
    HKU = "HKEY_USERS"
    HKCC = "HKEY_CURRENT_CONFIG"


class WinregType(IntEnum):
    REG_NONE = winreg.REG_NONE
    REG_SZ = winreg.REG_SZ
    REG_EXPAND_SZ = winreg.REG_EXPAND_SZ
    REG_BINARY = winreg.REG_BINARY
    REG_DWORD = winreg.REG_DWORD
    REG_DWORD_LITTLE_ENDIAN = winreg.REG_DWORD_LITTLE_ENDIAN
    REG_DWORD_BIG_ENDIAN = winreg.REG_DWORD_BIG_ENDIAN
    REG_LINK = winreg.REG_LINK
    REG_MULTI_SZ = winreg.REG_MULTI_SZ
    REG_RESOURCE_LIST = winreg.REG_RESOURCE_LIST
    REG_FULL_RESOURCE_DESCRIPTOR = winreg.REG_FULL_RESOURCE_DESCRIPTOR
    REG_RESOURCE_REQUIREMENTS_LIST = winreg.REG_RESOURCE_REQUIREMENTS_LIST
    REG_QWORD = winreg.REG_QWORD
    REG_QWORD_LITTLE_ENDIAN = winreg.REG_QWORD_LITTLE_ENDIAN



from winregistry.robot.keywords import Keywords

ROBOT_LIBRARY_SCOPE = "GLOBAL"


class robot(Keywords):  # pylint: disable=invalid-name
    """
    Minimalist Python library aimed at working with Windows Registry.
    https://github.com/shpaker/winregistry
        Examples:
        | ${path}               | HKLM\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run |
        | Write Registry Entry  | ${path}           | Notepad | notepad.exe               |
        | ${autorun} =          | Read Registry Key | ${path} |                           |
        | Log                   | ${autorun}        |         |                           |
        | Delete Registry Entry | ${path}           | Notepad |                           |
    """



from winregistry.consts import ShortRootAlias, WinregType
from winregistry.models import RegEntry, RegKey
from winregistry.winregistry import WinRegistry

__all__ = (
    "WinRegistry",
    "RegEntry",
    "RegKey",
    "WinregType",
    "ShortRootAlias",
)



from datetime import datetime, timedelta
from typing import Any, Optional, Union
from winreg import (
    KEY_READ,
    KEY_SET_VALUE,
    KEY_WRITE,
    ConnectRegistry,
    CreateKeyEx,
    DeleteKey,
    DeleteValue,
    EnumKey,
    EnumValue,
    HKEYType,
    OpenKey,
    QueryInfoKey,
    QueryValueEx,
    SetValueEx,
)

from winregistry.consts import WinregType
from winregistry.models import RegEntry, RegKey
from winregistry.utils import get_access_key, parse_path


class WinRegistry:
    def __init__(
        self,
        host: Optional[str] = None,
    ) -> None:
        self.host: Optional[str] = host
        self._client: Optional[HKEYType] = None
        self._handler = None
        self._root: Optional[HKEYType] = None

    def _get_handler(
        self,
        key: str,
        access: int,
        key_wow64_32key: bool,
    ) -> HKEYType:
        root, path = parse_path(key)
        access_key = get_access_key(access, key_wow64_32key)
        if not self._client or root != self._root:
            self._client = ConnectRegistry(self.host, root)
        key_handle = OpenKey(
            key=self._client,
            sub_key=path,
            reserved=0,
            access=access_key,
        )
        return key_handle

    def close(self) -> None:
        if self._client:
            self._client.Close()

    def __enter__(self) -> "WinRegistry":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):  # type: ignore
        self.close()
        if exc_val:
            raise

    def read_entry(
        self,
        reg_key: str,
        name: str,
        key_wow64_32key: bool = False,
    ) -> RegEntry:
        handle = self._get_handler(reg_key, KEY_READ, key_wow64_32key)
        raw_value, raw_type = QueryValueEx(handle, name)
        return RegEntry(
            reg_key=reg_key,
            name=name,
            value=raw_value,
            type=WinregType(raw_type),
            host=self.host,
        )

    def write_entry(
        self,
        reg_key: str,
        name: str,
        value: Any = None,
        reg_type: Union[WinregType, int] = WinregType.REG_SZ,
        key_wow64_32key: bool = False,
    ) -> None:
        if isinstance(reg_type, int):
            reg_type = WinregType(reg_type)
        handle = self._get_handler(reg_key, KEY_SET_VALUE, key_wow64_32key)
        SetValueEx(handle, name, 0, reg_type.value, value)

    def delete_entry(
        self,
        key: str,
        name: str,
        key_wow64_32key: bool = False,
    ) -> None:
        handle = self._get_handler(key, KEY_SET_VALUE, key_wow64_32key)
        DeleteValue(handle, name)

    def read_key(
        self,
        name: str,
        key_wow64_32key: bool = False,
    ) -> RegKey:
        handle = self._get_handler(name, KEY_READ, key_wow64_32key)
        keys_num, values_num, modify = QueryInfoKey(handle)
        modify_at = datetime(1601, 1, 1) + timedelta(microseconds=modify / 10)
        keys = list()
        entries = list()
        for key_i in range(0, keys_num):
            keys.append(EnumKey(handle, key_i))
        for key_i in range(0, values_num):
            entry_name, value, raw_type = EnumValue(handle, key_i)
            entries.append(
                RegEntry(
                    reg_key=name,
                    name=entry_name,
                    value=value,
                    type=WinregType(raw_type),
                    host=self.host,
                )
            )
        return RegKey(
            name=name,
            reg_keys=keys,
            entries=entries,
            modify_at=modify_at,
        )

    def create_key(
        self,
        name: str,
        key_wow64_32key: bool = False,
    ) -> None:
        handler = None
        sub_keys = name.split("\\")
        i = 0
        while i < len(sub_keys) and not handler:
            try:
                current = "\\".join(sub_keys[: len(sub_keys) - i])
                handler = self._get_handler(current, KEY_WRITE, key_wow64_32key)
            except FileNotFoundError:
                i += 1
        before_index = len(sub_keys) - i
        tail = "\\".join(sub_keys[before_index:])
        CreateKeyEx(
            key=handler,  # type: ignore
            sub_key=tail,
            reserved=0,
            access=get_access_key(KEY_WRITE),
        )

    def delete_key(
        self,
        name: str,
        key_wow64_32key: bool = False,
    ) -> None:
        parental, key_name = name.rsplit(sep="\\", maxsplit=1)
        handle = self._get_handler(parental, KEY_WRITE, key_wow64_32key)
        DeleteKey(handle, key_name)

    def delete_key_tree(
        self,
        name: str,
        key_wow64_32key: bool = False,
    ) -> None:
        handle = self._get_handler(name, KEY_READ, key_wow64_32key)
        keys_num, values_num, modify = QueryInfoKey(handle)  # pylint: disable=unused-variable
        for key_i in range(0, keys_num):
            key = EnumKey(handle, key_i)
            self.delete_key_tree(f"{name}\\{key}", key_wow64_32key)
        handle.Close()
        self.delete_key(name, key_wow64_32key)



import winreg
from typing import Tuple
from winreg import KEY_WOW64_32KEY, KEY_WOW64_64KEY, HKEYType

from winregistry import ShortRootAlias


def expand_short_root(
    root: str,
) -> str:
    try:
        root = ShortRootAlias[root].value
    except KeyError:
        pass
    return root


def get_access_key(
    access: int,
    key_wow64_32key: bool = False,
) -> int:
    x64_key = KEY_WOW64_32KEY if key_wow64_32key else KEY_WOW64_64KEY
    return access | x64_key


def parse_path(
    path: str,
) -> Tuple[HKEYType, str]:
    _root, key_path = path.split("\\", maxsplit=1)
    _root = expand_short_root(_root.upper())
    reg_root = getattr(winreg, _root)
    if not key_path:
        raise ValueError('Not found key in "{}"'.format(path))
    return reg_root, key_path



from dataclasses import dataclass
from datetime import datetime
from typing import Any, List, Optional

from winregistry.consts import WinregType


@dataclass(frozen=True)
class RegEntry:
    name: str
    reg_key: str
    value: Any
    type: WinregType
    host: Optional[str] = None


@dataclass(frozen=True)
class RegKey:
    name: str
    reg_keys: List[str]
    entries: List[RegEntry]
    modify_at: datetime



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['robot', 'winregistry', 'winregistry.robot']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'winregistry',
    'version': '1.1.1',
    'description': 'Library aimed at working with Windows registry',
    'long_description': '# winregistry\n\nMinimalist Python library aimed at working with Windows Registry.\n\n## Installation\n\n```bash\npip install winregistry\n```\n\n## Usage\n\n```py\nfrom winregistry import WinRegistry\n\nTEST_REG_PATH = r"HKLM\\SOFTWARE\\_REMOVE_ME_"\n\n\nif __name__ == "__main__":\n    with WinRegistry() as client:\n        client.create_key(TEST_REG_PATH)\n        client.write_entry(TEST_REG_PATH, "remove_me", "test")\n        test_entry = client.read_entry(TEST_REG_PATH, "remove_me")\n        assert test_entry.value == "test"\n        client.delete_entry(TEST_REG_PATH, "remove_me")\n```\n\nUsage with ``Robot Testing Framework`` Library\n----------------------------------------------\n\n```\n*** Settings ***\nLibrary    winregistry.robot\n\n*** Test Cases ***\nValid Login\n        ${path} =    Set Variable    HKLM\\\\SOFTWARE\\\\Microsoft\\\\Windows\\\\CurrentVersion\\\\Run\n        Write Registry Entry    ${path}             Notepad   notepad.exe\n        ${autorun} =            Read Registry Key   ${path}\n        Delete Registry Entry   ${path}             Notepad\n```\n',
    'author': 'Aleksandr Shpak',
    'author_email': 'shpaker@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/shpaker/winregistry',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
